@@ -5,14 +5,16 @@ import platform
 from os.path import expanduser, exists, dirname
 from configobj import ConfigObj
 
+
 def config_location():
-    if platform.system() == 'Windows':
-        return os.getenv('USERPROFILE') + '\\AppData\\Local\\dbcli\\pgcli\\'
-    elif 'XDG_CONFIG_HOME' in os.environ:
+    if 'XDG_CONFIG_HOME' in os.environ:
         return '%s/pgcli/' % expanduser(os.environ['XDG_CONFIG_HOME'])
+    elif platform.system() == 'Windows':
+        return os.getenv('USERPROFILE') + '\\AppData\\Local\\dbcli\\pgcli\\'
     else:
         return expanduser('~/.config/pgcli/')
 
+
 def load_config(usr_cfg, def_cfg=None):
     cfg = ConfigObj()
     cfg.merge(ConfigObj(def_cfg, interpolation=False))
@@ -41,6 +43,7 @@ def write_default_config(source, destination, overwrite=False):
 
     shutil.copyfile(source, destination)
 
+
 def upgrade_config(config, def_config):
     cfg = load_config(config, def_config)
     cfg.write()
