@@ -3,7 +3,7 @@ import logging
 import re
 from itertools import count, repeat, chain
 import operator
-from collections import namedtuple, defaultdict
+from collections import namedtuple, defaultdict, OrderedDict
 from pgspecial.namedqueries import NamedQueries
 from prompt_toolkit.completion import Completer, Completion
 from prompt_toolkit.contrib.completers import PathCompleter
@@ -18,28 +18,37 @@ from .packages.pgliterals.main import get_literals
 from .packages.prioritization import PrevalenceCounter
 from .config import load_config, config_location
 
-try:
-    from collections import OrderedDict
-except ImportError:
-    from .packages.ordereddict import OrderedDict
-
 _logger = logging.getLogger(__name__)
 
 NamedQueries.instance = NamedQueries.from_config(
     load_config(config_location() + 'config'))
 
 Match = namedtuple('Match', ['completion', 'priority'])
-_SchemaObject = namedtuple('SchemaObject', ['name', 'schema', 'function'])
+
+_SchemaObject = namedtuple('SchemaObject', 'name schema meta')
 
 
-def SchemaObject(name, schema=None, function=False):
-    return _SchemaObject(name, schema, function)
+def SchemaObject(name, schema=None, meta=None):
+    return _SchemaObject(name, schema, meta)
+
 
 _Candidate = namedtuple(
-    'Candidate', ['completion', 'prio', 'meta', 'synonyms', 'prio2']
+    'Candidate', 'completion prio meta synonyms prio2 display'
 )
-def Candidate(completion, prio=None, meta=None, synonyms=None, prio2=None):
-    return _Candidate(completion, prio, meta, synonyms or [completion], prio2)
+
+
+def Candidate(
+        completion, prio=None, meta=None, synonyms=None, prio2=None,
+        display=None
+):
+    return _Candidate(
+        completion, prio, meta, synonyms or [completion], prio2,
+        display or completion
+    )
+
+
+# Used to strip trailing '::some_type' from default-value expressions
+arg_default_type_strip_regex = re.compile(r'::[\w\.]+(\[\])?$')
 
 normalize_ref = lambda ref: ref if ref[0] == '"' else '"' + ref.lower() +  '"'
 
@@ -60,6 +69,7 @@ class PGCompleter(Completer):
     keywords = tuple(set(chain(keywords_tree.keys(), *keywords_tree.values())))
     functions = get_literals('functions')
     datatypes = get_literals('datatypes')
+    reserved_words = set(get_literals('reserved'))
 
     def __init__(self, smart_completion=True, pgspecial=None, settings=None):
         super(PGCompleter, self).__init__()
@@ -67,6 +77,16 @@ class PGCompleter(Completer):
         self.pgspecial = pgspecial
         self.prioritizer = PrevalenceCounter()
         settings = settings or {}
+        self.signature_arg_style = settings.get(
+            'signature_arg_style', '{arg_name} {arg_type}'
+        )
+        self.call_arg_style = settings.get(
+            'call_arg_style', '{arg_name: <{max_arg_len}} := {arg_default}'
+        )
+        self.call_arg_display_style = settings.get(
+            'call_arg_display_style', '{arg_name}'
+        )
+        self.call_arg_oneliner_max = settings.get('call_arg_oneliner_max', 2)
         self.search_path_filter = settings.get('search_path_filter')
         self.generate_aliases = settings.get('generate_aliases')
         self.casing_file = settings.get('casing_file')
@@ -86,11 +106,7 @@ class PGCompleter(Completer):
         if keyword_casing not in ('upper', 'lower', 'auto'):
             keyword_casing = 'upper'
         self.keyword_casing = keyword_casing
-
-        self.reserved_words = set()
-        for x in self.keywords:
-            self.reserved_words.update(x.split())
-        self.name_pattern = re.compile("^[_a-z][_a-z0-9\$]*$")
+        self.name_pattern = re.compile(r"^[_a-z][_a-z0-9\$]*$")
 
         self.databases = []
         self.dbmetadata = {'tables': {}, 'views': {}, 'functions': {},
@@ -122,7 +138,6 @@ class PGCompleter(Completer):
         return [self.escape_name(name) for name in names]
 
     def extend_database_names(self, databases):
-        databases = self.escaped_names(databases)
         self.databases.extend(databases)
 
     def extend_keywords(self, additional_keywords):
@@ -201,8 +216,6 @@ class PGCompleter(Completer):
     def extend_functions(self, func_data):
 
         # func_data is a list of function metadata namedtuples
-        # with fields schema_name, func_name, arg_list, result,
-        # is_aggregate, is_window, is_set_returning
 
         # dbmetadata['schema_name']['functions']['function_name'] should return
         # the function metadata namedtuple for the corresponding function
@@ -218,6 +231,22 @@ class PGCompleter(Completer):
 
             self.all_completions.add(func)
 
+        self._refresh_arg_list_cache()
+
+    def _refresh_arg_list_cache(self):
+        # We keep a cache of {function_usage:{function_metadata: function_arg_list_string}}
+        # This is used when suggesting functions, to avoid the latency that would result
+        # if we'd recalculate the arg lists each time we suggest functions (in large DBs)
+        self._arg_list_cache = {
+            usage: {
+                meta: self._arg_list(meta, usage)
+                for sch, funcs in self.dbmetadata['functions'].items()
+                for func, metas in funcs.items()
+                for meta in metas
+            }
+            for usage in ('call', 'call_display', 'signature')
+        }
+
     def extend_foreignkeys(self, fk_data):
 
         # fk_data is a list of ForeignKey namedtuples, with fields
@@ -344,7 +373,7 @@ class PGCompleter(Completer):
         matches = []
         for cand in collection:
             if isinstance(cand, _Candidate):
-                item, prio, display_meta, synonyms, prio2 = cand
+                item, prio, display_meta, synonyms, prio2, display = cand
                 if display_meta is None:
                     display_meta = meta
                 syn_matches = (_match(x) for x in synonyms)
@@ -352,7 +381,7 @@ class PGCompleter(Completer):
                 syn_matches = [m for m in syn_matches if m]
                 sort_key = max(syn_matches) if syn_matches else None
             else:
-                item, display_meta, prio, prio2 = cand, meta, 0, 0
+                item, display_meta, prio, prio2, display = cand, meta, 0, 0, cand
                 sort_key = _match(cand)
 
             if sort_key:
@@ -374,15 +403,22 @@ class PGCompleter(Completer):
                     + tuple(c for c in item))
 
                 item = self.case(item)
+                display = self.case(display)
                 priority = (
                     sort_key, type_priority, prio, priority_func(item),
                     prio2, lexical_priority
                 )
-
-                matches.append(Match(
-                    completion=Completion(item, -text_len,
-                    display_meta=display_meta),
-                    priority=priority))
+                matches.append(
+                    Match(
+                        completion=Completion(
+                            text=item,
+                            start_position=-text_len,
+                            display_meta=display_meta,
+                            display=display
+                        ),
+                        priority=priority
+                    )
+                )
         return matches
 
     def case(self, word):
@@ -420,7 +456,6 @@ class PGCompleter(Completer):
 
         return [m.completion for m in matches]
 
-
     def get_column_matches(self, suggestion, word_before_cursor):
         tables = suggestion.table_refs
         do_qualify = suggestion.qualifiable and {'always': True, 'never': False,
@@ -602,31 +637,34 @@ class PGCompleter(Completer):
         return self.find_matches(word_before_cursor, conds, meta='join')
 
     def get_function_matches(self, suggestion, word_before_cursor, alias=False):
-        def _cand(func, alias):
-            return self._make_cand(func, alias, suggestion)
-        if suggestion.filter == 'for_from_clause':
+        if suggestion.usage == 'from':
             # Only suggest functions allowed in FROM clause
-            filt = lambda f: not f.is_aggregate and not f.is_window
-            funcs = [_cand(f, alias)
-                     for f in self.populate_functions(suggestion.schema, filt)]
+            def filt(f): return not f.is_aggregate and not f.is_window
         else:
-            fs = self.populate_schema_objects(suggestion.schema, 'functions')
-            funcs = [_cand(f, alias=False) for f in fs]
+            alias = False
 
+            def filt(f): return True
+        arg_mode = {
+            'signature': 'signature',
+            'special': None,
+        }.get(suggestion.usage, 'call')
         # Function overloading means we way have multiple functions of the same
         # name at this point, so keep unique names only
-        funcs = set(funcs)
+        funcs = set(
+            self._make_cand(f, alias, suggestion, arg_mode)
+            for f in self.populate_functions(suggestion.schema, filt)
+        )
 
-        funcs = self.find_matches(word_before_cursor, funcs, meta='function')
+        matches = self.find_matches(word_before_cursor, funcs, meta='function')
 
-        if not suggestion.schema and not suggestion.filter:
+        if not suggestion.schema and not suggestion.usage:
             # also suggest hardcoded functions using startswith matching
             predefined_funcs = self.find_matches(
                 word_before_cursor, self.functions, mode='strict',
                 meta='function')
-            funcs.extend(predefined_funcs)
+            matches.extend(predefined_funcs)
 
-        return funcs
+        return matches
 
     def get_schema_matches(self, suggestion, word_before_cursor):
         schema_names = self.dbmetadata['tables'].keys()
@@ -648,23 +686,85 @@ class PGCompleter(Completer):
         s = suggestion
         t_sug = Table(s.schema, s.table_refs, s.local_tables)
         v_sug = View(s.schema, s.table_refs)
-        f_sug = Function(s.schema, s.table_refs, filter='for_from_clause')
-        return (self.get_table_matches(t_sug, word_before_cursor, alias)
+        f_sug = Function(s.schema, s.table_refs, usage='from')
+        return (
+            self.get_table_matches(t_sug, word_before_cursor, alias)
             + self.get_view_matches(v_sug, word_before_cursor, alias)
-            + self.get_function_matches(f_sug, word_before_cursor, alias))
+            + self.get_function_matches(f_sug, word_before_cursor, alias)
+        )
 
-    # Note: tbl is a SchemaObject
-    def _make_cand(self, tbl, do_alias, suggestion):
+    def _arg_list(self, func, usage):
+        """Returns a an arg list string, e.g. `(_foo:=23)` for a func.
+
+        :param func is a FunctionMetadata object
+        :param usage is 'call', 'call_display' or 'signature'
+
+        """
+        template = {
+            'call':  self.call_arg_style,
+            'call_display': self.call_arg_display_style,
+            'signature': self.signature_arg_style
+        }[usage]
+        args = func.args()
+        if not template:
+            return '()'
+        elif usage == 'call' and len(args) < 2:
+            return '()'
+        elif usage == 'call' and func.has_variadic():
+            return '()'
+        multiline = usage == 'call' and len(args) > self.call_arg_oneliner_max
+        max_arg_len = max(len(a.name) for a in args) if multiline else 0
+        args = (
+            self._format_arg(template, arg, arg_num + 1, max_arg_len)
+            for arg_num, arg in enumerate(args)
+        )
+        if multiline:
+            return '(' + ','.join('\n    ' + a for a in args if a) + '\n)'
+        else:
+            return '(' + ', '.join(a for a in args if a) + ')'
+
+    def _format_arg(self, template, arg, arg_num, max_arg_len):
+        if not template:
+            return None
+        if arg.has_default:
+            arg_default = 'NULL' if arg.default is None else arg.default
+            # Remove trailing ::(schema.)type
+            arg_default = arg_default_type_strip_regex.sub('', arg_default)
+        else:
+            arg_default = ''
+        return template.format(
+            max_arg_len=max_arg_len,
+            arg_name=self.case(arg.name),
+            arg_num=arg_num,
+            arg_type=arg.datatype,
+            arg_default=arg_default
+        )
+
+    def _make_cand(self, tbl, do_alias, suggestion, arg_mode=None):
+        """Returns a Candidate namedtuple.
+
+        :param tbl is a SchemaObject
+        :param arg_mode determines what type of arg list to suffix for functions.
+        Possible values: call, signature
+
+        """
         cased_tbl = self.case(tbl.name)
         if do_alias:
             alias = self.alias(cased_tbl, suggestion.table_refs)
         synonyms = (cased_tbl, generate_alias(cased_tbl))
-        maybe_parens = '()' if tbl.function else ''
         maybe_alias = (' ' + alias) if do_alias else ''
         maybe_schema = (self.case(tbl.schema) + '.') if tbl.schema else ''
-        item = maybe_schema + cased_tbl + maybe_parens + maybe_alias
+        suffix = self._arg_list_cache[arg_mode][tbl.meta] if arg_mode else ''
+        if arg_mode == 'call':
+            display_suffix = self._arg_list_cache['call_display'][tbl.meta]
+        elif arg_mode == 'signature':
+            display_suffix = self._arg_list_cache['signature'][tbl.meta]
+        else:
+            display_suffix = ''
+        item = maybe_schema + cased_tbl + suffix + maybe_alias
+        display = maybe_schema + cased_tbl + display_suffix + maybe_alias
         prio2 = 0 if tbl.schema else 1
-        return Candidate(item, synonyms=synonyms, prio2=prio2)
+        return Candidate(item, synonyms=synonyms, prio2=prio2, display=display)
 
     def get_table_matches(self, suggestion, word_before_cursor, alias=False):
         tables = self.populate_schema_objects(suggestion.schema, 'tables')
@@ -772,10 +872,12 @@ class PGCompleter(Completer):
     }
 
     def populate_scoped_cols(self, scoped_tbls, local_tbls=()):
-        """ Find all columns in a set of scoped_tables
+        """Find all columns in a set of scoped_tables.
+
         :param scoped_tbls: list of TableReference namedtuples
         :param local_tbls: tuple(TableMetadata)
         :return: {TableReference:{colname:ColumnMetaData}}
+
         """
         ctes = dict((normalize_ref(t.name), t.columns) for t in local_tbls)
         columns = OrderedDict()
@@ -816,8 +918,10 @@ class PGCompleter(Completer):
         return columns
 
     def _get_schemas(self, obj_typ, schema):
-        """ Returns a list of schemas from which to suggest objects
-        schema is the schema qualification input by the user (if any)
+        """Returns a list of schemas from which to suggest objects.
+
+        :param schema is the schema qualification input by the user (if any)
+
         """
         metadata = self.dbmetadata[obj_typ]
         if schema:
@@ -829,26 +933,28 @@ class PGCompleter(Completer):
         return None if parent or schema in self.search_path else schema
 
     def populate_schema_objects(self, schema, obj_type):
-        """Returns a list of SchemaObjects representing tables, views, funcs
-        schema is the schema qualification input by the user (if any)
+        """Returns a list of SchemaObjects representing tables or views.
+
+        :param schema is the schema qualification input by the user (if any)
+
         """
 
         return [
             SchemaObject(
                 name=obj,
-                schema=(self._maybe_schema(schema=sch, parent=schema)),
-                function=(obj_type == 'functions')
+                schema=(self._maybe_schema(schema=sch, parent=schema))
             )
             for sch in self._get_schemas(obj_type, schema)
             for obj in self.dbmetadata[obj_type][sch].keys()
         ]
 
     def populate_functions(self, schema, filter_func):
-        """Returns a list of function names
+        """Returns a list of function SchemaObjects.
+
+        :param filter_func is a function that accepts a FunctionMetadata
+        namedtuple and returns a boolean indicating whether that
+        function should be kept or discarded
 
-        filter_func is a function that accepts a FunctionMetadata namedtuple
-        and returns a boolean indicating whether that function should be
-        kept or discarded
         """
 
         # Because of multiple dispatch, we can have multiple functions
@@ -858,7 +964,7 @@ class PGCompleter(Completer):
             SchemaObject(
                 name=func,
                 schema=(self._maybe_schema(schema=sch, parent=schema)),
-                function=True
+                meta=meta
             )
             for sch in self._get_schemas('functions', schema)
             for (func, metas) in self.dbmetadata['functions'][sch].items()
