@@ -156,6 +156,12 @@ class PGExecute(object):
         FROM pg_catalog.pg_database d
         ORDER BY 1'''
 
+    socket_directory_query = '''
+        SELECT setting
+        FROM pg_settings
+        WHERE name = 'unix_socket_directories'
+    '''
+
     def __init__(self, database, user, password, host, port, dsn, **kwargs):
         self.dbname = database
         self.user = user
@@ -203,9 +209,11 @@ class PGExecute(object):
             # When we connect using a DSN, we don't really know what db,
             # user, etc. we connected to. Let's read it.
             # Note: moved this after setting autocommit because of #664.
-            db, user, host, port = self._select_one(
-                cursor,
-                'select current_database(), current_user, inet_server_addr(), inet_server_port()')
+            dsn_parameters = conn.get_dsn_parameters()
+            db = dsn_parameters['dbname']
+            user = dsn_parameters['user']
+            host = dsn_parameters['host']
+            port = dsn_parameters['port']
 
         self.dbname = db
         self.user = user
@@ -213,6 +221,9 @@ class PGExecute(object):
         self.host = host
         self.port = port
 
+        if not self.host:
+            self.host = self.get_socket_directory()
+
         cursor.execute("SHOW ALL")
         db_parameters = dict(name_val_desc[:2] for name_val_desc in cursor.fetchall())
 
@@ -334,7 +345,8 @@ class PGExecute(object):
 
         """
         return (isinstance(e, psycopg2.OperationalError) and
-                psycopg2.errorcodes.lookup(e.pgcode) != 'LOCK_NOT_AVAILABLE')
+                (not e.pgcode or
+                 psycopg2.errorcodes.lookup(e.pgcode) != 'LOCK_NOT_AVAILABLE'))
 
     def execute_normal_sql(self, split_sql):
         """Returns tuple (title, rows, headers, status)"""
@@ -489,6 +501,14 @@ class PGExecute(object):
             headers = [x[0] for x in cur.description]
             return cur.fetchall(), headers, cur.statusmessage
 
+    def get_socket_directory(self):
+        with self.conn.cursor() as cur:
+            _logger.debug('Socket directory Query. sql: %r',
+                          self.socket_directory_query)
+            cur.execute(self.socket_directory_query)
+            result = cur.fetchone()
+            return result[0] if result else ''
+
     def foreignkeys(self):
         """Yields ForeignKey named tuples"""
 
@@ -542,7 +562,8 @@ class PGExecute(object):
                         prorettype::regtype::text return_type,
                         p.proisagg is_aggregate,
                         p.proiswindow is_window,
-                        p.proretset is_set_returning
+                        p.proretset is_set_returning,
+                        pg_get_expr(proargdefaults, 0) AS arg_defaults
                 FROM pg_catalog.pg_proc p
                         INNER JOIN pg_catalog.pg_namespace n
                             ON n.oid = p.pronamespace
@@ -559,7 +580,8 @@ class PGExecute(object):
                         prorettype::regtype::text,
                         p.proisagg is_aggregate,
                         false is_window,
-                        p.proretset is_set_returning
+                        p.proretset is_set_returning,
+                        NULL AS arg_defaults
                 FROM pg_catalog.pg_proc p
                 INNER JOIN pg_catalog.pg_namespace n
                 ON n.oid = p.pronamespace
@@ -576,7 +598,8 @@ class PGExecute(object):
                         '' ret_type,
                         p.proisagg is_aggregate,
                         false is_window,
-                        p.proretset is_set_returning
+                        p.proretset is_set_returning,
+                        NULL AS arg_defaults
                 FROM pg_catalog.pg_proc p
                 INNER JOIN pg_catalog.pg_namespace n
                 ON n.oid = p.pronamespace
@@ -639,7 +662,7 @@ class PGExecute(object):
     def casing(self):
         """Yields the most common casing for names used in db functions"""
         with self.conn.cursor() as cur:
-            query = '''
+            query = r'''
           WITH Words AS (
                 SELECT regexp_split_to_table(prosrc, '\W+') AS Word, COUNT(1)
                 FROM pg_catalog.pg_proc P
@@ -671,6 +694,9 @@ class PGExecute(object):
                 UNION -- Schema names
                 SELECT nspname
                 FROM pg_catalog.pg_namespace
+                UNION -- Parameter names
+                SELECT unnest(proargnames)
+                FROM pg_proc
             )
             SELECT Word
             FROM OrderWords
